<?php
# Jisko: An open-source microblogging application
# Copyright (C) 2010 Rubén Díaz <outime@gmail.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

function do_header($page = '') {
	
	if(defined('NO_GUI')) return $page;
	
	global $globals;
	global $_USER;
	global $_PROFILE;
	global $gettext_tables;
		
	shuffle($globals['logo']);
	
	if (file_exists(PATH.'includes/languages/'.LANG.'/LC_MESSAGES/messages.mo')) {
		$gettext_tables = new gettext_reader(
			new CachedFileReader(PATH.'includes/languages/'.LANG.'/LC_MESSAGES/messages.mo')
		);
    	$gettext_tables->load_tables();
	}
	
	$tex = '';
	$tex .= '
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="'.LANG.'" lang="'.LANG.'">
<head>

<title>'.TITLE.' '.SEPARATOR.' '.NAME.'</title>

<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<link rel="stylesheet" href="'.BASE.'themes/';
	if(!empty($_USER['theme'])) {
		$tex .= $_USER['theme'];
	} elseif(!empty($_PROFILE['theme'])) {
		$tex .= $_PROFILE['theme'];
	} else {
		$tex .= THEME;
	}
	$tex .= '/css/styles.css" type="text/css" />
<link rel="stylesheet" href="'.BASE.'static/css/common.css" type="text/css" />
<link rel="shortcut icon" href="'.BASE.'static/img/favicon.png" type="image/png" />
<meta name="keywords" content="'.str_replace('%name', NAME, str_replace('%language', LANG, META_KEYWORDS)).'" />
<meta name="robots" content="'.META_ROBOTS.'" />
<meta name="description" content="'.str_replace('%name', NAME, META_DESCRIPTION).'" />
<script src="'.BASE.'static/js/jquery.js" type="text/javascript"></script>
<script src="'.BASE.'static/js/jsoc.js" type="text/javascript"></script>
<script src="'.BASE.'static/js/tooltip.php" type="text/javascript"></script>
<script type="text/javascript">var timerID = 0;</script>';
	$ajax = defined('AJAX_SECTION');
	if($ajax) {
		$ajax_section = AJAX_SECTION;
		$ajax_page = AJAX_PAGE;
		$tex .= '<script src="'.BASE.'static/js/general.php?section='.$ajax_section.'&amp;page='.$ajax_page.'" type="text/javascript"></script>';	
	} else {
		$tex .= '<script src="'.BASE.'static/js/general.php" type="text/javascript"></script>';
	}

	$tex .= checkBackground();
	$tex .= '</head>';
	$tex .= '<body>';
	if($ajax) $tex .= '<script type="text/javascript">var timerID = setInterval(\'reloadNotes()\', 30000);</script>';
	$tex .= '<div id="loading"><img src="'.BASE.'static/img/smileys/cry.png" alt="cry" /></div>'; // pre-load of icons
	$tex .= '
<div class="web">
<div class="header">
<a href="'.BASE.'" class="logo">
<img src="'.BASE.'static/img/logos/'.$globals['logo'][0].'" alt="'.NAME.'" /> 
<img src="'.BASE.'static/img/logos/logo_jisky.png" alt="Jisky" />
</a>';
	if ($_USER) {
		$tex .= '
<div class="menu">
<a href="'.BASE.'home">'.__('Home').'</a> | <a href="'.BASE.'public">'.__('Public notes').'</a> | ';

	foreach ($globals['menubar_links'] as $key=>$value) {
		$tex .= '<a href="'.$value.'">'.$key.'</a> | ';
	}

	$tex .= '<a href="'.BASE.'settings">'.__('Settings').'</a> | <a href="'.BASE.'logout">'.__('Logout').'</a>';
	} else {
		$tex .= '
<div class="menu_ul">
<a href="'.BASE.'home">'.__('Home').'</a> | <a href="'.BASE.'public">'.__('Public notes').'</a>';

		foreach ($globals['menubar_links'] as $key=>$value) {
			$tex .= ' | <a href="'.$value.'">'.$key.'</a>';
		}

	}
	$tex .= '
</div>
</div>
<div class="border_top"></div>
<div class="content">
<div class="content_top">
<div class="content_left_top">';
	$tex = $tex.$page;
	return $tex;
}

function do_side() {
	echo '
</div>
<div class="content_right_top">';
	import('sidebar');
}

function do_footer() {
	global $db;
	global $globals;
	if(defined('NO_GUI')) return;
	
	do_side();
	echo '
</div>
<div class="separator"></div>
</div>
</div>
<div class="footer">
<div class="search">
<form method="post" action="'.BASE.'search">
<p><input name="info" class="search_user" value="'.__('Search...').'" onblur="if(this.value==\'\') this.value=\''.__('Search...').'\';" onfocus="if(this.value==\''.__('Search...').'\') this.value=\'\';" type="text" />
<input type="image" src="'.BASE.'static/img/icons/search.png" class="search_icon" alt="'.__('Search').'" /></p>
</form>
</div> ('.$db->queries.') | ';
if (defined('BLOG')) echo '<a href="'.BLOG.'">'.__('Blog').'</a> | ';
if (defined('WIKI')) echo '<a href="'.WIKI.'">'.__('Wiki').'</a> | ';
if (defined('TOS') && (TOS == true)) echo '<a href="'.BASE.'tos">'.__('Terms of Service').'</a> | ';
echo '<a href="'.BASE.'faq">'.__('FAQ').'</a> | ';
if (defined('PUBLIC_ADMIN_MAIL') && (PUBLIC_ADMIN_MAIL == true)) echo '<a href="mailto:'.str_replace('@', '[at]', ADMIN_MAIL).'">'.__('Contact').'</a> | ';

	foreach ($globals['footer_links'] as $key=>$value) {
		echo '<a href="'.$value.'">'.$key.'</a> | ';
	}


echo '<a href="'.BASE.'mobile"><img src="'.BASE.'static/img/icons/phone.png" alt="phone"/> '.__('Mobile interface').'</a> | Powered by <a href="http://jisko.org/">Jisko</a>'."\n".'
</div>
</div>';

	if(defined('SC_PROJECT'))
		echo '
<!-- Start of StatCounter Code -->
<script type="text/javascript">
sc_project='.SC_PROJECT.'; 
sc_invisible=1; 
sc_partition='.SC_PARTITION.'; 
sc_security="'.SC_SECURITY.'"; 
</script>

<script type="text/javascript" src="http://www.statcounter.com/counter/counter_xhtml.js"></script><noscript><div class="statcounter"><a class="statcounter" href="http://www.statcounter.com/free_hit_counter.html"><img class="statcounter" src="'.SC_URL.'" alt="statcounter"/></a></div></noscript>
<!-- End of StatCounter Code -->';

echo '</body>
</html>';
}